#!/usr/bin/perl
#
#  This program interfaces with our modem and collects 
#  callerID information.
#
#  by Anthony Anderberg - 8.15.2001
#

$html_file = '/usr/local/www/data/callerid/index.html';
$log_file = '/usr/local/home/callerid/rawdata.log';
$| = 1;  # autoflush

$foo = '';

# Stuff for our clients
@client_list = ('192.168.1.21', '192.168.1.22');
use Socket;
socket(SOCKET, PF_INET, SOCK_DGRAM, getprotobyname("udp")) or die "$!";

#
# Our Log File
#
open (LOG, ">>$log_file");

#
# Make sure our log files are autoflushed
#
$old_fh = select(LOG);
$| = 1;
select($old_fh);

#
# Tell the modem to print out caller ID information
#
open (MODEM, ">/dev/cuaa1");
print MODEM "at#CID=1\n\r";
close (MODEM);

#
# Open the device
#
open (DATAIN, "/dev/cuaa1");

#
# Attempt to read data from the device filehandle 
# one byte at a time.
#
$hold_date = '';
$hold_time = '';
$hold_number = '';

#
# My Zoom modem prints "\r\n" at the end of each line.
# (Carriage Return - Line Feed instead of our default 
# line splitter which is "\n" only.)
#
$/ = "\r\n";

while (<DATAIN>)
 {
   # print everything that comes in to the log file
   print LOG $_;

   #
   # Now we process the collected line of data
   #

   # Whack off the \r\n at the end of every line
   # (including the blank ones!)
   $_ = substr($_, 0, -2);

   # Omit the blank lines and those that just say RING
   if (($_)&&($_ ne 'RING'))
     {
      ($type, $value) = split(/ = /, $_);

      # If the first part of the line says DATE we're 
      # looking at the begining of a record.
      # print it in the form MM-DD
      if ($type eq 'DATE')
        {
         $tmp = substr($value, 0, 2)."-".substr($value, 2, 2);
         $hold_date = $tmp;
        }

      # TIME is the second line, print it in the format HH:MM
      elsif ($type eq 'TIME')
        {
         $tmp = substr($value, 0, 2).":".substr($value, 2, 2);
         $hold_time = $tmp;
        }

      # NMBR is the third line, its either the number or "0"
      elsif ($type eq 'NMBR')
        {
         $hold_number = $value;
        }

      # If the name is sent in the record we get a NAME 
      # field as the last line.
      elsif ($type eq 'NAME')
        {
         $hold_name = $value;
         ProcessRecord();
        }

      # If the name isn't sent in the record we seem to 
      # get a MESG field instead as the last line.
      elsif ($type eq 'MESG')
        {
         if ($value eq '08014F')
           {
            $tmp = "Unavailable";
           }
         elsif ($value eq '080150')
           {
            $tmp = "Private (Blocked)";
           }
         else
           {
            $tmp = $value;
           }
       
         $hold_name = $tmp;
         ProcessRecord();
        }

      # An unknown field, ignore it and its value 
      # and hope for the best.
      else
        {
        }
    } # end of if
 } # end of while()
close (DATAIN);

close (LOG);

#########################################
sub ProcessRecord
 { 
   # The delimiter ";;-;;" should never appear in the data stream 
   # and is easy to spot.
   $web_record = "<TR><TD>$hold_date $hold_time</TD><TD>$hold_number</TD><TD>$hold_name</TD></TR>\n";
   $send_record = "$hold_date $hold_time;;-;;$hold_number;;-;;$hold_name";

   foreach $host (@client_list)
    {
     $ipaddr = inet_aton("$host");
     $portaddr = sockaddr_in("200", $ipaddr);
     send(SOCKET, $send_record, 0, $portaddr)==length($send_record);
    }

   # Suck in the current file
   open (WEBPAGE, "$html_file");
   undef $/; # Treat the file as one big scalar instead of multiple lines...
   $current_file = <WEBPAGE>;
   close (WEBPAGE);
   $/ = "\r\n"; # Back to the line splitter for our modem.

   # Split the page in two
   ($page_start, $page_end) = split(/<!-- Next Record Goes Here -->\n/, $current_file, 2);

   # Create the new page
   $new_page = $page_start."<!-- Next Record Goes Here -->\n".$web_record.$page_end;
   open (NEWPAGE, ">$html_file");
   print NEWPAGE $new_page;
   close (NEWPAGE);
 }
